!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ###########################
      MODULE MODD_CH_EMIS_FIELD_n
!     ###########################
!
!!****  *MODD_CH_EMIS_FIELD_n* - declaration of chemical emission data arrays
!!
!!    PURPOSE
!!    -------
!       The purpose of this declarative module is to specify  the 
!     chemical emission data arrays.
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!      
!!
!!    AUTHOR
!!    ------
!!	D. Gazen   *L.A.*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    08/03/2001                      
!!      01/12/03    (D.Gazen) change emissions handling for surf. externalization
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_TYPE_EFUTIL
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
TYPE CH_EMIS_FIELD_t
!
  REAL               :: XTIME_SIMUL  = 0.
  INTEGER            :: NEMIS_NBR
!                          ! number of chemical pgd fields chosen by user
  CHARACTER(LEN=3) , DIMENSION(:), POINTER :: CEMIS_AREA
!                          ! areas where chemical pgd fields are defined
!                          ! 'ALL' : everywhere
!                          ! 'SEA' : where sea exists
!                          ! 'LAN' : where land exists
!                          ! 'WAT' : where inland water exists
!                          ! 'NAT' : where natural or agricultural areas exist
!                          ! 'TWN' : where town areas exist
!                          ! 'STR' : where streets are present
!                          ! 'BLD' : where buildings are present
!                          !
  CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_COMMENT ! comment
  CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_NAME
!                          ! name of the chemical pgd fields (emitted species)
!
  INTEGER,           DIMENSION(:), POINTER :: NEMIS_TIME   ! emission time
!
  REAL,              DIMENSION(:,:), POINTER:: XEMIS_FIELDS ! emission pgd fields values
!
  INTEGER                                          :: NEMISPEC_NBR ! Number of chemical species
!
  TYPE(EMISSVAR_T),  DIMENSION(:), POINTER :: TSEMISS      ! Offline emission struct array
!
  TYPE(PRONOSVAR_T),               POINTER     :: TSPRONOSLIST ! Head pointer on pronostic
!                                                              variables list
!-------------------------------------------------------------------------------
!
END TYPE CH_EMIS_FIELD_t

TYPE(CH_EMIS_FIELD_t), ALLOCATABLE, TARGET, SAVE :: CH_EMIS_FIELD_MODEL(:)

INTEGER, POINTER :: NEMIS_NBR=>NULL()
!$OMP THREADPRIVATE(NEMIS_NBR)
REAL,    POINTER :: XTIME_SIMUL=>NULL()
!$OMP THREADPRIVATE(XTIME_SIMUL)
 CHARACTER(LEN=3) , DIMENSION(:), POINTER :: CEMIS_AREA=>NULL()
!$OMP THREADPRIVATE(CEMIS_AREA)
 CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_COMMENT=>NULL()
!$OMP THREADPRIVATE(CEMIS_COMMENT)
 CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_NAME=>NULL()
!$OMP THREADPRIVATE(CEMIS_NAME)
INTEGER,           DIMENSION(:), POINTER :: NEMIS_TIME=>NULL()
!$OMP THREADPRIVATE(NEMIS_TIME)
REAL,              DIMENSION(:,:), POINTER:: XEMIS_FIELDS=>NULL()
!$OMP THREADPRIVATE(XEMIS_FIELDS)
INTEGER, POINTER :: NEMISPEC_NBR=>NULL()
!$OMP THREADPRIVATE(NEMISPEC_NBR)
TYPE(EMISSVAR_T),  DIMENSION(:), POINTER :: TSEMISS=>NULL()
!$OMP THREADPRIVATE(TSEMISS)
TYPE(PRONOSVAR_T),               POINTER     :: TSPRONOSLIST=>NULL()
!$OMP THREADPRIVATE(TSPRONOSLIST)

CONTAINS

SUBROUTINE CH_EMIS_FIELD_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
CH_EMIS_FIELD_MODEL(KFROM)%CEMIS_AREA=>CEMIS_AREA
CH_EMIS_FIELD_MODEL(KFROM)%CEMIS_COMMENT=>CEMIS_COMMENT
CH_EMIS_FIELD_MODEL(KFROM)%CEMIS_NAME=>CEMIS_NAME
CH_EMIS_FIELD_MODEL(KFROM)%NEMIS_TIME=>NEMIS_TIME
CH_EMIS_FIELD_MODEL(KFROM)%XEMIS_FIELDS=>XEMIS_FIELDS
CH_EMIS_FIELD_MODEL(KFROM)%TSEMISS=>TSEMISS
CH_EMIS_FIELD_MODEL(KFROM)%TSPRONOSLIST=>TSPRONOSLIST
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_GOTO_MODEL',0,ZHOOK_HANDLE)

XTIME_SIMUL=>CH_EMIS_FIELD_MODEL(KTO)%XTIME_SIMUL
NEMIS_NBR=>CH_EMIS_FIELD_MODEL(KTO)%NEMIS_NBR
CEMIS_AREA=>CH_EMIS_FIELD_MODEL(KTO)%CEMIS_AREA
CEMIS_COMMENT=>CH_EMIS_FIELD_MODEL(KTO)%CEMIS_COMMENT
CEMIS_NAME=>CH_EMIS_FIELD_MODEL(KTO)%CEMIS_NAME
NEMIS_TIME=>CH_EMIS_FIELD_MODEL(KTO)%NEMIS_TIME
XEMIS_FIELDS=>CH_EMIS_FIELD_MODEL(KTO)%XEMIS_FIELDS
NEMISPEC_NBR=>CH_EMIS_FIELD_MODEL(KTO)%NEMISPEC_NBR
TSEMISS=>CH_EMIS_FIELD_MODEL(KTO)%TSEMISS
TSPRONOSLIST=>CH_EMIS_FIELD_MODEL(KTO)%TSPRONOSLIST
IF (LHOOK) CALL DR_HOOK('MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE CH_EMIS_FIELD_GOTO_MODEL

SUBROUTINE CH_EMIS_FIELD_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(CH_EMIS_FIELD_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%CEMIS_AREA)
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%CEMIS_COMMENT)
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%CEMIS_NAME)
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%NEMIS_TIME)
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%XEMIS_FIELDS)
  NULLIFY(CH_EMIS_FIELD_MODEL(J)%TSEMISS)
ENDDO
CH_EMIS_FIELD_MODEL(:)%XTIME_SIMUL=0.
CH_EMIS_FIELD_MODEL(:)%NEMIS_NBR=0
CH_EMIS_FIELD_MODEL(:)%NEMISPEC_NBR=0
IF (LHOOK) CALL DR_HOOK("MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE CH_EMIS_FIELD_ALLOC

SUBROUTINE CH_EMIS_FIELD_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(CH_EMIS_FIELD_MODEL)) DEALLOCATE(CH_EMIS_FIELD_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_CH_EMIS_FIELD_N:CH_EMIS_FIELD_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE CH_EMIS_FIELD_DEALLO

END MODULE MODD_CH_EMIS_FIELD_n

