!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!########################
MODULE MODD_DIAG_EVAP_ISBA_n
!########################
!
!!****  *MODD_DIAG_ISBA - declaration of packed surface parameters for ISBA scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/11/03
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_EVAP_ISBA_t
!------------------------------------------------------------------------------
!
  LOGICAL :: LSURF_EVAP_BUDGET   ! flag for all terms of evaporation
  LOGICAL :: LSURF_BUDGETC       ! flag for surface cumulated energy budget
  LOGICAL :: LRESET_BUDGETC      ! flag for surface cumulated energy budget
  LOGICAL :: LWATER_BUDGET       ! flag for isba water budget including input  
                                 ! fluxes (rain and snow) and reservoir tendencies
!
!* variables for each patch
!
  REAL, POINTER, DIMENSION(:,:) :: XLEG          ! latent heat of evaporation over the ground   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEGI         ! surface soil ice sublimation                 (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEV          ! latent heat of evaporation over vegetation   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLES          ! latent heat of sublimation over the snow     (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLESL         ! latent heat of evaporation over the snow     (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLER          ! evaporation from canopy water interception   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLETR         ! evapotranspiration of the vegetation         (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XEVAP         ! evapotranspiration                           (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDRAIN        ! soil drainage flux                           (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XRUNOFF       ! sub-grid and supersaturation runoff          (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XHORT         ! sub-grid Horton runoff from the SGH scheme   (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XRRVEG        !  precipitation intercepted by the vegetation (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XMELT         ! snow melt                                    (kg/m2/s)      
  REAL, POINTER, DIMENSION(:,:) :: XIFLOOD       ! Floodplains infiltration                     (kg/m2/s)      
  REAL, POINTER, DIMENSION(:,:) :: XPFLOOD       ! Precipitation intercepted by the floodplains (kg/m2/s)      
  REAL, POINTER, DIMENSION(:,:) :: XLE_FLOOD     ! Floodplains evapotration                     (W/m2)      
  REAL, POINTER, DIMENSION(:,:) :: XLEI_FLOOD    ! Floodplains evapotration                     (W/m2)      
  REAL, POINTER, DIMENSION(:,:) :: XDRIP         ! dripping from the vegetation reservoir       (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XIRRIG_FLUX   ! irrigation rate (as soil input)              (kg/m2/s)
!  
  REAL, POINTER, DIMENSION(:,:) :: XGPP          ! Gross Primary Production                     (kgCO2/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XRESP_AUTO    ! Autotrophic respiration                      (kgCO2/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XRESP_ECO     ! Ecosystem respiration                        (kgCO2/m2/s)
!
  REAL, POINTER, DIMENSION(:,:) :: XDWG          ! liquid soil moisture time tendencies         (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XDWGI         ! solid soil moisture time tendencies          (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XDWR          ! canopy water time tendencies                 (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XDSWE         ! snow water equivalent time tendencies        (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XWATBUD       ! ISBA water budget                            (kg/m2/s)
!
!* average variables
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEG      ! latent heat of evaporation over the ground   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEGI     ! surface soil ice sublimation                 (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEV      ! latent heat of evaporation over vegetation   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LES      ! latent heat of sublimation over the snow     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LESL     ! latent heat of evaporation over the snow     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LER      ! evaporation from canopy water interception   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LETR     ! evapotranspiration of the vegetation         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_EVAP     ! evapotranspiration                           (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DRAIN    ! soil drainage flux                           (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RUNOFF   ! sub-grid and supersaturation runoff          (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HORT     ! sub-grid Horton runoff from the SGH scheme   (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DRIP     ! dripping from the vegetation reservoir       (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_MELT     ! snow melt                                    (kg/m2/s)      
  REAL, POINTER, DIMENSION(:)   :: XAVG_IFLOOD   ! Floodplains infiltration                     (kg/m2/s)      
  REAL, POINTER, DIMENSION(:)   :: XAVG_PFLOOD   ! Precipitation intercepted by the floodplains (kg/m2/s)      
  REAL, POINTER, DIMENSION(:)   :: XAVG_LE_FLOOD ! Floodplains evapotration                     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEI_FLOOD! Floodplains evapotration                     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RRVEG    ! precipitation intercepted by the vegetation  (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_IRRIG_FLUX! irrigation rate (as soil input)             (kg/m2/s)
!  
  REAL, POINTER, DIMENSION(:)   :: XAVG_GPP      ! Gross Primary Production                     (kgCO2/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RESP_AUTO! Autotrophic respiration                      (kgCO2/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RESP_ECO ! Ecosystem respiration                        (kgCO2/m2/s)
!
  REAL, POINTER, DIMENSION(:)   :: XRAINFALL     ! input rainfall rate for LWATER_BUDGET        (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XSNOWFALL     ! input snowfall rate for LWATER_BUDGET        (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWG      ! liquid soil moisture time tendencies         (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWGI     ! solid soil moisture time tendencies          (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWR      ! canopy water time tendencies                 (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DSWE     ! snow water equivalent time tendencies        (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_WATBUD   ! ISBA water budget                            (kg/m2/s)
!  
!* budget summation variables for each patch
!
  REAL, POINTER, DIMENSION(:,:) :: XRNC          ! net radiation at surface                     (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XHC           ! sensible heat flux                           (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEC          ! total latent heat flux                       (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEIC         ! sublimation latent heat flux                 (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XGFLUXC       ! net soil-vegetation flux                     (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEGC         ! latent heat of evaporation over the ground   (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEGIC        ! surface soil ice sublimation                 (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLEVC         ! latent heat of evaporation over vegetation   (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLESC         ! latent heat of sublimation over the snow     (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLESLC        ! latent heat of evaporation over the snow     (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLERC         ! evaporation from canopy water interception   (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLETRC        ! evapotranspiration of the vegetation         (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XEVAPC        ! evapotranspiration                           (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDRAINC       ! soil drainage flux                           (kg/m2) 
  REAL, POINTER, DIMENSION(:,:) :: XRUNOFFC      ! sub-grid and supersaturation runoff          (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XHORTC        ! sub-grid Horton runoff from the SGH scheme   (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDRIPC        ! dripping from the vegetation reservoir       (kg/m2/s)
  REAL, POINTER, DIMENSION(:,:) :: XMELTC        ! snow melt                                    (kg/m2)      
  REAL, POINTER, DIMENSION(:,:) :: XIFLOODC      ! Floodplains infiltration                     (kg/m2)      
  REAL, POINTER, DIMENSION(:,:) :: XPFLOODC      ! Precipitation intercepted by the floodplains (kg/m2)      
  REAL, POINTER, DIMENSION(:,:) :: XLE_FLOODC    ! Floodplains evapotration                     (J/m2)  
  REAL, POINTER, DIMENSION(:,:) :: XLEI_FLOODC   ! Floodplains evapotration                     (J/m2)  
  REAL, POINTER, DIMENSION(:,:) :: XICEFLUXC     ! ice calving flux                             (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XRRVEGC       ! precipitation intercepted by the vegetation  (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XIRRIG_FLUXC  ! irrigation rate (as soil input)              (kg/m2)
!  
  REAL, POINTER, DIMENSION(:,:) :: XGPPC         ! Gross Primary Production                     (kgCO2/m2)
  REAL, POINTER, DIMENSION(:,:) :: XRESPC_AUTO   ! Autotrophic respiration                      (kgCO2/m2)
  REAL, POINTER, DIMENSION(:,:) :: XRESPC_ECO    ! Ecosystem respiration                        (kgCO2/m2)
!
  REAL, POINTER, DIMENSION(:,:) :: XDWGC         ! liquid soil moisture time tendencies         (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDWGIC        ! solid soil moisture time tendencies          (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDWRC         ! canopy water time tendencies                 (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDSWEC        ! snow water equivalent time tendencies        (kg/m2)
  REAL, POINTER, DIMENSION(:,:) :: XWATBUDC      ! ISBA water budget                            (kg/m2)
!
!* average budget summation variables
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_RNC       ! net radiation at surface                     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HC        ! sensible heat flux                           (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEC       ! total latent heat flux                       (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEIC      ! sublimation latent heat flux                 (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_GFLUXC    ! net soil-vegetation flux                     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEGC      ! latent heat of evaporation over the ground   (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEGIC     ! surface soil ice sublimation                 (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEVC      ! latent heat of evaporation over vegetation   (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LESC      ! latent heat of sublimation over the snow     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LESLC     ! latent heat of evaporation over the snow     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LERC      ! evaporation from canopy water interception   (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LETRC     ! evapotranspiration of the vegetation         (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_EVAPC     ! evapotranspiration                           (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DRAINC    ! soil drainage flux                           (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RUNOFFC   ! sub-grid and supersaturation runoff          (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HORTC     ! sub-grid Horton runoff from the SGH scheme   (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DRIPC     ! dripping from the vegetation reservoir       (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_MELTC     ! snow melt                                    (kg/m2)   
  REAL, POINTER, DIMENSION(:)   :: XAVG_IFLOODC   ! Floodplains infiltration                     (kg/m2)      
  REAL, POINTER, DIMENSION(:)   :: XAVG_PFLOODC   ! Precipitation intercepted by the floodplains (kg/m2)      
  REAL, POINTER, DIMENSION(:)   :: XAVG_LE_FLOODC ! Floodplains evapotration                     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEI_FLOODC! Floodplains evapotration                     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_ICEFLUXC  ! ice calving flux                             (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RRVEGC    ! precipitation intercepted by the vegetation  (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_IRRIG_FLUXC! irrigation rate (as soil input)             (kg/m2)
!  
  REAL, POINTER, DIMENSION(:)   :: XAVG_GPPC      ! Gross Primary Production                     (kgCO2/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RESPC_AUTO! Autotrophic respiration                      (kgCO2/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RESPC_ECO ! Ecosystem respiration                        (kgCO2/m2)  
!
  REAL, POINTER, DIMENSION(:)   :: XRAINFALLC     ! input rainfall rate for LWATER_BUDGET        (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XSNOWFALLC     ! input snowfall rate for LWATER_BUDGET        (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWGC      ! liquid soil moisture time tendencies         (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWGIC     ! solid soil moisture time tendencies          (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DWRC      ! canopy water time tendencies                 (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_DSWEC     ! snow water equivalent time tendencies        (kg/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_WATBUDC   ! ISBA water budget                            (kg/m2)
! 
!------------------------------------------------------------------------------
!

END TYPE DIAG_EVAP_ISBA_t

TYPE(DIAG_EVAP_ISBA_t), ALLOCATABLE, TARGET, SAVE :: DIAG_EVAP_ISBA_MODEL(:)

LOGICAL, POINTER :: LSURF_EVAP_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_EVAP_BUDGET)
LOGICAL, POINTER :: LSURF_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGETC)
LOGICAL, POINTER :: LRESET_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LRESET_BUDGETC)
LOGICAL, POINTER :: LWATER_BUDGET=>NULL()
!$OMP THREADPRIVATE(LWATER_BUDGET)
REAL, POINTER, DIMENSION(:,:) :: XLEG=>NULL()
!$OMP THREADPRIVATE(XLEG)
REAL, POINTER, DIMENSION(:,:) :: XLEGI=>NULL()
!$OMP THREADPRIVATE(XLEGI)
REAL, POINTER, DIMENSION(:,:) :: XLEV=>NULL()
!$OMP THREADPRIVATE(XLEV)
REAL, POINTER, DIMENSION(:,:) :: XLES=>NULL()
!$OMP THREADPRIVATE(XLES)
REAL, POINTER, DIMENSION(:,:) :: XLESL=>NULL()
!$OMP THREADPRIVATE(XLESL)
REAL, POINTER, DIMENSION(:,:) :: XLER=>NULL()
!$OMP THREADPRIVATE(XLER)
REAL, POINTER, DIMENSION(:,:) :: XLETR=>NULL()
!$OMP THREADPRIVATE(XLETR)
REAL, POINTER, DIMENSION(:,:) :: XEVAP=>NULL()
!$OMP THREADPRIVATE(XEVAP)
REAL, POINTER, DIMENSION(:,:) :: XDRAIN=>NULL()
!$OMP THREADPRIVATE(XDRAIN)
REAL, POINTER, DIMENSION(:,:) :: XRUNOFF=>NULL()
!$OMP THREADPRIVATE(XRUNOFF)
REAL, POINTER, DIMENSION(:,:) :: XHORT=>NULL()
!$OMP THREADPRIVATE(XHORT)
REAL, POINTER, DIMENSION(:,:) :: XDRIP=>NULL()
!$OMP THREADPRIVATE(XDRIP)
REAL, POINTER, DIMENSION(:,:) :: XMELT=>NULL()
!$OMP THREADPRIVATE(XMELT)
REAL, POINTER, DIMENSION(:,:) :: XIFLOOD=>NULL()
!$OMP THREADPRIVATE(XIFLOOD)
REAL, POINTER, DIMENSION(:,:) :: XPFLOOD=>NULL()
!$OMP THREADPRIVATE(XPFLOOD)
REAL, POINTER, DIMENSION(:,:) :: XLE_FLOOD=>NULL()
!$OMP THREADPRIVATE(XLE_FLOOD)
REAL, POINTER, DIMENSION(:,:) :: XLEI_FLOOD=>NULL()
!$OMP THREADPRIVATE(XLEI_FLOOD)
REAL, POINTER, DIMENSION(:,:) :: XRRVEG=>NULL()
!$OMP THREADPRIVATE(XRRVEG)
REAL, POINTER, DIMENSION(:,:) :: XIRRIG_FLUX=>NULL()
!$OMP THREADPRIVATE(XIRRIG_FLUX)
REAL, POINTER, DIMENSION(:,:) :: XGPP=>NULL()
!$OMP THREADPRIVATE(XGPP)
REAL, POINTER, DIMENSION(:,:) :: XRESP_AUTO=>NULL()
!$OMP THREADPRIVATE(XRESP_AUTO)
REAL, POINTER, DIMENSION(:,:) :: XRESP_ECO=>NULL()
!$OMP THREADPRIVATE(XRESP_ECO)
REAL, POINTER, DIMENSION(:,:) :: XDWG=>NULL()
!$OMP THREADPRIVATE(XDWG)
REAL, POINTER, DIMENSION(:,:) :: XDWGI=>NULL()
!$OMP THREADPRIVATE(XDWGI)
REAL, POINTER, DIMENSION(:,:) :: XDWR=>NULL()
!$OMP THREADPRIVATE(XDWR)
REAL, POINTER, DIMENSION(:,:) :: XDSWE=>NULL()
!$OMP THREADPRIVATE(XDSWE)
REAL, POINTER, DIMENSION(:,:) :: XWATBUD=>NULL()
!$OMP THREADPRIVATE(XWATBUD)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEG=>NULL()
!$OMP THREADPRIVATE(XAVG_LEG)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEGI=>NULL()
!$OMP THREADPRIVATE(XAVG_LEGI)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEV=>NULL()
!$OMP THREADPRIVATE(XAVG_LEV)
REAL, POINTER, DIMENSION(:)   :: XAVG_LES=>NULL()
!$OMP THREADPRIVATE(XAVG_LES)
REAL, POINTER, DIMENSION(:)   :: XAVG_LESL=>NULL()
!$OMP THREADPRIVATE(XAVG_LESL)
REAL, POINTER, DIMENSION(:)   :: XAVG_LER=>NULL()
!$OMP THREADPRIVATE(XAVG_LER)
REAL, POINTER, DIMENSION(:)   :: XAVG_LETR=>NULL()
!$OMP THREADPRIVATE(XAVG_LETR)
REAL, POINTER, DIMENSION(:)   :: XAVG_EVAP=>NULL()
!$OMP THREADPRIVATE(XAVG_EVAP)
REAL, POINTER, DIMENSION(:)   :: XAVG_DRAIN=>NULL()
!$OMP THREADPRIVATE(XAVG_DRAIN)
REAL, POINTER, DIMENSION(:)   :: XAVG_RUNOFF=>NULL()
!$OMP THREADPRIVATE(XAVG_RUNOFF)
REAL, POINTER, DIMENSION(:)   :: XAVG_HORT=>NULL()
!$OMP THREADPRIVATE(XAVG_HORT)
REAL, POINTER, DIMENSION(:)   :: XAVG_DRIP=>NULL()
!$OMP THREADPRIVATE(XAVG_DRIP)
REAL, POINTER, DIMENSION(:)   :: XAVG_MELT=>NULL()
!$OMP THREADPRIVATE(XAVG_MELT)
REAL, POINTER, DIMENSION(:)   :: XAVG_IFLOOD=>NULL()
!$OMP THREADPRIVATE(XAVG_IFLOOD)
REAL, POINTER, DIMENSION(:)   :: XAVG_PFLOOD=>NULL()
!$OMP THREADPRIVATE(XAVG_PFLOOD)
REAL, POINTER, DIMENSION(:)   :: XAVG_LE_FLOOD=>NULL()
!$OMP THREADPRIVATE(XAVG_LE_FLOOD)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEI_FLOOD=>NULL()
!$OMP THREADPRIVATE(XAVG_LEI_FLOOD)
REAL, POINTER, DIMENSION(:)   :: XAVG_RRVEG=>NULL()
!$OMP THREADPRIVATE(XAVG_RRVEG)
REAL, POINTER, DIMENSION(:)   :: XAVG_IRRIG_FLUX=>NULL()
!$OMP THREADPRIVATE(XAVG_IRRIG_FLUX)
REAL, POINTER, DIMENSION(:)   :: XAVG_GPP=>NULL()
!$OMP THREADPRIVATE(XAVG_GPP)
REAL, POINTER, DIMENSION(:)   :: XAVG_RESP_AUTO=>NULL()
!$OMP THREADPRIVATE(XAVG_RESP_AUTO)
REAL, POINTER, DIMENSION(:)   :: XAVG_RESP_ECO=>NULL()
!$OMP THREADPRIVATE(XAVG_RESP_ECO)
REAL, POINTER, DIMENSION(:)   :: XRAINFALL=>NULL()
!$OMP THREADPRIVATE(XRAINFALL)
REAL, POINTER, DIMENSION(:)   :: XSNOWFALL=>NULL()
!$OMP THREADPRIVATE(XSNOWFALL)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWG=>NULL()
!$OMP THREADPRIVATE(XAVG_DWG)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWGI=>NULL()
!$OMP THREADPRIVATE(XAVG_DWGI)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWR=>NULL()
!$OMP THREADPRIVATE(XAVG_DWR)
REAL, POINTER, DIMENSION(:)   :: XAVG_DSWE=>NULL()
!$OMP THREADPRIVATE(XAVG_DSWE)
REAL, POINTER, DIMENSION(:)   :: XAVG_WATBUD=>NULL()
!$OMP THREADPRIVATE(XAVG_WATBUD)
REAL, POINTER, DIMENSION(:,:) :: XRNC=>NULL()
!$OMP THREADPRIVATE(XRNC)
REAL, POINTER, DIMENSION(:,:) :: XHC=>NULL()
!$OMP THREADPRIVATE(XHC)
REAL, POINTER, DIMENSION(:,:) :: XLEC=>NULL()
!$OMP THREADPRIVATE(XLEC)
REAL, POINTER, DIMENSION(:,:) :: XLEIC=>NULL()
!$OMP THREADPRIVATE(XLEIC)
REAL, POINTER, DIMENSION(:,:) :: XGFLUXC=>NULL()
!$OMP THREADPRIVATE(XGFLUXC)
REAL, POINTER, DIMENSION(:,:) :: XLEGC=>NULL()
!$OMP THREADPRIVATE(XLEGC)
REAL, POINTER, DIMENSION(:,:) :: XLEGIC=>NULL()
!$OMP THREADPRIVATE(XLEGIC)
REAL, POINTER, DIMENSION(:,:) :: XLEVC=>NULL()
!$OMP THREADPRIVATE(XLEVC)
REAL, POINTER, DIMENSION(:,:) :: XLESC=>NULL()
!$OMP THREADPRIVATE(XLESC)
REAL, POINTER, DIMENSION(:,:) :: XLESLC=>NULL()
!$OMP THREADPRIVATE(XLESLC)
REAL, POINTER, DIMENSION(:,:) :: XLERC=>NULL()
!$OMP THREADPRIVATE(XLERC)
REAL, POINTER, DIMENSION(:,:) :: XLETRC=>NULL()
!$OMP THREADPRIVATE(XLETRC)
REAL, POINTER, DIMENSION(:,:) :: XEVAPC=>NULL()
!$OMP THREADPRIVATE(XEVAPC)
REAL, POINTER, DIMENSION(:,:) :: XDRAINC=>NULL()
!$OMP THREADPRIVATE(XDRAINC)
REAL, POINTER, DIMENSION(:,:) :: XRUNOFFC=>NULL()
!$OMP THREADPRIVATE(XRUNOFFC)
REAL, POINTER, DIMENSION(:,:) :: XHORTC=>NULL()
!$OMP THREADPRIVATE(XHORTC)
REAL, POINTER, DIMENSION(:,:) :: XDRIPC=>NULL()
!$OMP THREADPRIVATE(XDRIPC)
REAL, POINTER, DIMENSION(:,:) :: XMELTC=>NULL()
!$OMP THREADPRIVATE(XMELTC)
REAL, POINTER, DIMENSION(:,:) :: XIFLOODC=>NULL()
!$OMP THREADPRIVATE(XIFLOODC)
REAL, POINTER, DIMENSION(:,:) :: XPFLOODC=>NULL()
!$OMP THREADPRIVATE(XPFLOODC)
REAL, POINTER, DIMENSION(:,:) :: XLE_FLOODC=>NULL()
!$OMP THREADPRIVATE(XLE_FLOODC)
REAL, POINTER, DIMENSION(:,:) :: XLEI_FLOODC=>NULL()
!$OMP THREADPRIVATE(XLEI_FLOODC)
REAL, POINTER, DIMENSION(:,:) :: XICEFLUXC=>NULL()
!$OMP THREADPRIVATE(XICEFLUXC)
REAL, POINTER, DIMENSION(:,:) :: XRRVEGC=>NULL()
!$OMP THREADPRIVATE(XRRVEGC)
REAL, POINTER, DIMENSION(:,:) :: XIRRIG_FLUXC=>NULL()
!$OMP THREADPRIVATE(XIRRIG_FLUXC)
REAL, POINTER, DIMENSION(:,:) :: XGPPC=>NULL()
!$OMP THREADPRIVATE(XGPPC)
REAL, POINTER, DIMENSION(:,:) :: XRESPC_AUTO=>NULL()
!$OMP THREADPRIVATE(XRESPC_AUTO)
REAL, POINTER, DIMENSION(:,:) :: XRESPC_ECO=>NULL()
!$OMP THREADPRIVATE(XRESPC_ECO)
REAL, POINTER, DIMENSION(:,:) :: XDWGC=>NULL()
!$OMP THREADPRIVATE(XDWGC)
REAL, POINTER, DIMENSION(:,:) :: XDWGIC=>NULL()
!$OMP THREADPRIVATE(XDWGIC)
REAL, POINTER, DIMENSION(:,:) :: XDWRC=>NULL()
!$OMP THREADPRIVATE(XDWRC)
REAL, POINTER, DIMENSION(:,:) :: XDSWEC=>NULL()
!$OMP THREADPRIVATE(XDSWEC)
REAL, POINTER, DIMENSION(:,:) :: XWATBUDC=>NULL()
!$OMP THREADPRIVATE(XWATBUDC)
REAL, POINTER, DIMENSION(:)   :: XAVG_RNC=>NULL()
!$OMP THREADPRIVATE(XAVG_RNC)
REAL, POINTER, DIMENSION(:)   :: XAVG_HC=>NULL()
!$OMP THREADPRIVATE(XAVG_HC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEIC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEIC)
REAL, POINTER, DIMENSION(:)   :: XAVG_GFLUXC=>NULL()
!$OMP THREADPRIVATE(XAVG_GFLUXC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEGC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEGC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEGIC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEGIC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEVC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEVC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LESC=>NULL()
!$OMP THREADPRIVATE(XAVG_LESC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LESLC=>NULL()
!$OMP THREADPRIVATE(XAVG_LESLC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LERC=>NULL()
!$OMP THREADPRIVATE(XAVG_LERC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LETRC=>NULL()
!$OMP THREADPRIVATE(XAVG_LETRC)
REAL, POINTER, DIMENSION(:)   :: XAVG_EVAPC=>NULL()
!$OMP THREADPRIVATE(XAVG_EVAPC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DRAINC=>NULL()
!$OMP THREADPRIVATE(XAVG_DRAINC)
REAL, POINTER, DIMENSION(:)   :: XAVG_RUNOFFC=>NULL()
!$OMP THREADPRIVATE(XAVG_RUNOFFC)
REAL, POINTER, DIMENSION(:)   :: XAVG_HORTC=>NULL()
!$OMP THREADPRIVATE(XAVG_HORTC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DRIPC=>NULL()
!$OMP THREADPRIVATE(XAVG_DRIPC)
REAL, POINTER, DIMENSION(:)   :: XAVG_MELTC=>NULL()
!$OMP THREADPRIVATE(XAVG_MELTC)
REAL, POINTER, DIMENSION(:)   :: XAVG_IFLOODC=>NULL()
!$OMP THREADPRIVATE(XAVG_IFLOODC)
REAL, POINTER, DIMENSION(:)   :: XAVG_PFLOODC=>NULL()
!$OMP THREADPRIVATE(XAVG_PFLOODC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LE_FLOODC=>NULL()
!$OMP THREADPRIVATE(XAVG_LE_FLOODC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEI_FLOODC=>NULL()
!$OMP THREADPRIVATE(XAVG_LEI_FLOODC)
REAL, POINTER, DIMENSION(:)   :: XAVG_ICEFLUXC=>NULL()
!$OMP THREADPRIVATE(XAVG_ICEFLUXC)
REAL, POINTER, DIMENSION(:)   :: XAVG_RRVEGC=>NULL()
!$OMP THREADPRIVATE(XAVG_RRVEGC)
REAL, POINTER, DIMENSION(:)   :: XAVG_IRRIG_FLUXC=>NULL()
!$OMP THREADPRIVATE(XAVG_IRRIG_FLUXC)
REAL, POINTER, DIMENSION(:)   :: XAVG_GPPC=>NULL()
!$OMP THREADPRIVATE(XAVG_GPPC)
REAL, POINTER, DIMENSION(:)   :: XAVG_RESPC_AUTO=>NULL()
!$OMP THREADPRIVATE(XAVG_RESPC_AUTO)
REAL, POINTER, DIMENSION(:)   :: XAVG_RESPC_ECO=>NULL()
!$OMP THREADPRIVATE(XAVG_RESPC_ECO)
REAL, POINTER, DIMENSION(:)   :: XRAINFALLC=>NULL()
!$OMP THREADPRIVATE(XRAINFALLC)
REAL, POINTER, DIMENSION(:)   :: XSNOWFALLC=>NULL()
!$OMP THREADPRIVATE(XSNOWFALLC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWGC=>NULL()
!$OMP THREADPRIVATE(XAVG_DWGC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWGIC=>NULL()
!$OMP THREADPRIVATE(XAVG_DWGIC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DWRC=>NULL()
!$OMP THREADPRIVATE(XAVG_DWRC)
REAL, POINTER, DIMENSION(:)   :: XAVG_DSWEC=>NULL()
!$OMP THREADPRIVATE(XAVG_DSWEC)
REAL, POINTER, DIMENSION(:)   :: XAVG_WATBUDC=>NULL()
!$OMP THREADPRIVATE(XAVG_WATBUDC)
!
CONTAINS
!
SUBROUTINE DIAG_EVAP_ISBA_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEG=>XLEG
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEGI=>XLEGI
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEV=>XLEV
DIAG_EVAP_ISBA_MODEL(KFROM)%XLES=>XLES
DIAG_EVAP_ISBA_MODEL(KFROM)%XLESL=>XLESL
DIAG_EVAP_ISBA_MODEL(KFROM)%XLER=>XLER
DIAG_EVAP_ISBA_MODEL(KFROM)%XLETR=>XLETR
DIAG_EVAP_ISBA_MODEL(KFROM)%XEVAP=>XEVAP
DIAG_EVAP_ISBA_MODEL(KFROM)%XDRAIN=>XDRAIN
DIAG_EVAP_ISBA_MODEL(KFROM)%XRUNOFF=>XRUNOFF
DIAG_EVAP_ISBA_MODEL(KFROM)%XHORT=>XHORT
DIAG_EVAP_ISBA_MODEL(KFROM)%XDRIP=>XDRIP
DIAG_EVAP_ISBA_MODEL(KFROM)%XMELT=>XMELT
DIAG_EVAP_ISBA_MODEL(KFROM)%XIFLOOD=>XIFLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XPFLOOD=>XPFLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XLE_FLOOD=>XLE_FLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEI_FLOOD=>XLEI_FLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XRRVEG=>XRRVEG
DIAG_EVAP_ISBA_MODEL(KFROM)%XIRRIG_FLUX=>XIRRIG_FLUX
DIAG_EVAP_ISBA_MODEL(KFROM)%XGPP=>XGPP
DIAG_EVAP_ISBA_MODEL(KFROM)%XRESP_AUTO=>XRESP_AUTO
DIAG_EVAP_ISBA_MODEL(KFROM)%XRESP_ECO=>XRESP_ECO
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWG=>XDWG
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWGI=>XDWGI
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWR=>XDWR
DIAG_EVAP_ISBA_MODEL(KFROM)%XDSWE=>XDSWE
DIAG_EVAP_ISBA_MODEL(KFROM)%XWATBUD=>XWATBUD
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEG=>XAVG_LEG
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEGI=>XAVG_LEGI
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEV=>XAVG_LEV
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LES=>XAVG_LES
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LESL=>XAVG_LESL
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LER=>XAVG_LER
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LETR=>XAVG_LETR
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_EVAP=>XAVG_EVAP
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DRAIN=>XAVG_DRAIN
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RUNOFF=>XAVG_RUNOFF
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_HORT=>XAVG_HORT
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DRIP=>XAVG_DRIP
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_MELT=>XAVG_MELT
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_IFLOOD=>XAVG_IFLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_PFLOOD=>XAVG_PFLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LE_FLOOD=>XAVG_LE_FLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEI_FLOOD=>XAVG_LEI_FLOOD
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RRVEG=>XAVG_RRVEG
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_IRRIG_FLUX=>XAVG_IRRIG_FLUX
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_GPP=>XAVG_GPP
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RESP_AUTO=>XAVG_RESP_AUTO
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RESP_ECO=>XAVG_RESP_ECO
DIAG_EVAP_ISBA_MODEL(KFROM)%XRAINFALL=>XRAINFALL
DIAG_EVAP_ISBA_MODEL(KFROM)%XSNOWFALL=>XSNOWFALL
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWG=>XAVG_DWG
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWGI=>XAVG_DWGI
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWR=>XAVG_DWR
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DSWE=>XAVG_DSWE
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_WATBUD=>XAVG_WATBUD
DIAG_EVAP_ISBA_MODEL(KFROM)%XRNC=>XRNC
DIAG_EVAP_ISBA_MODEL(KFROM)%XHC=>XHC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEC=>XLEC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEIC=>XLEIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XGFLUXC=>XGFLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEGC=>XLEGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEGIC=>XLEGIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEVC=>XLEVC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLESC=>XLESC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLESLC=>XLESLC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLERC=>XLERC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLETRC=>XLETRC
DIAG_EVAP_ISBA_MODEL(KFROM)%XEVAPC=>XEVAPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XDRAINC=>XDRAINC
DIAG_EVAP_ISBA_MODEL(KFROM)%XRUNOFFC=>XRUNOFFC
DIAG_EVAP_ISBA_MODEL(KFROM)%XHORTC=>XHORTC
DIAG_EVAP_ISBA_MODEL(KFROM)%XDRIPC=>XDRIPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XMELTC=>XMELTC
DIAG_EVAP_ISBA_MODEL(KFROM)%XIFLOODC=>XIFLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XPFLOODC=>XPFLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLE_FLOODC=>XLE_FLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XLEI_FLOODC=>XLEI_FLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XICEFLUXC=>XICEFLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XRRVEGC=>XRRVEGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XIRRIG_FLUXC=>XIRRIG_FLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XGPPC=>XGPPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XRESPC_AUTO=>XRESPC_AUTO
DIAG_EVAP_ISBA_MODEL(KFROM)%XRESPC_ECO=>XRESPC_ECO
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWGC=>XDWGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWGIC=>XDWGIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XDWRC=>XDWRC
DIAG_EVAP_ISBA_MODEL(KFROM)%XDSWEC=>XDSWEC
DIAG_EVAP_ISBA_MODEL(KFROM)%XWATBUDC=>XWATBUDC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RNC=>XAVG_RNC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_HC=>XAVG_HC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEC=>XAVG_LEC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEIC=>XAVG_LEIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_GFLUXC=>XAVG_GFLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEGC=>XAVG_LEGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEGIC=>XAVG_LEGIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEVC=>XAVG_LEVC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LESC=>XAVG_LESC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LESLC=>XAVG_LESLC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LERC=>XAVG_LERC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LETRC=>XAVG_LETRC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_EVAPC=>XAVG_EVAPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DRAINC=>XAVG_DRAINC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RUNOFFC=>XAVG_RUNOFFC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_HORTC=>XAVG_HORTC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DRIPC=>XAVG_DRIPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_MELTC=>XAVG_MELTC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_IFLOODC=>XAVG_IFLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_PFLOODC=>XAVG_PFLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LE_FLOODC=>XAVG_LE_FLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_LEI_FLOODC=>XAVG_LEI_FLOODC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_ICEFLUXC=>XAVG_ICEFLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RRVEGC=>XAVG_RRVEGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_IRRIG_FLUXC=>XAVG_IRRIG_FLUXC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_GPPC=>XAVG_GPPC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RESPC_AUTO=>XAVG_RESPC_AUTO
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_RESPC_ECO=>XAVG_RESPC_ECO
DIAG_EVAP_ISBA_MODEL(KFROM)%XRAINFALLC=>XRAINFALLC
DIAG_EVAP_ISBA_MODEL(KFROM)%XSNOWFALLC=>XSNOWFALLC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWGC=>XAVG_DWGC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWGIC=>XAVG_DWGIC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DWRC=>XAVG_DWRC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_DSWEC=>XAVG_DSWEC
DIAG_EVAP_ISBA_MODEL(KFROM)%XAVG_WATBUDC=>XAVG_WATBUDC
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_GOTO_MODEL',0,ZHOOK_HANDLE)
LSURF_EVAP_BUDGET=>DIAG_EVAP_ISBA_MODEL(KTO)%LSURF_EVAP_BUDGET
LSURF_BUDGETC=>DIAG_EVAP_ISBA_MODEL(KTO)%LSURF_BUDGETC
LRESET_BUDGETC=>DIAG_EVAP_ISBA_MODEL(KTO)%LRESET_BUDGETC
LWATER_BUDGET=>DIAG_EVAP_ISBA_MODEL(KTO)%LWATER_BUDGET
XLEG=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEG
XLEGI=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEGI
XLEV=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEV
XLES=>DIAG_EVAP_ISBA_MODEL(KTO)%XLES
XLESL=>DIAG_EVAP_ISBA_MODEL(KTO)%XLESL
XLER=>DIAG_EVAP_ISBA_MODEL(KTO)%XLER
XLETR=>DIAG_EVAP_ISBA_MODEL(KTO)%XLETR
XEVAP=>DIAG_EVAP_ISBA_MODEL(KTO)%XEVAP
XDRAIN=>DIAG_EVAP_ISBA_MODEL(KTO)%XDRAIN
XRUNOFF=>DIAG_EVAP_ISBA_MODEL(KTO)%XRUNOFF
XHORT=>DIAG_EVAP_ISBA_MODEL(KTO)%XHORT
XDRIP=>DIAG_EVAP_ISBA_MODEL(KTO)%XDRIP
XMELT=>DIAG_EVAP_ISBA_MODEL(KTO)%XMELT
XIFLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XIFLOOD
XPFLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XPFLOOD
XLE_FLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XLE_FLOOD
XLEI_FLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEI_FLOOD
XRRVEG=>DIAG_EVAP_ISBA_MODEL(KTO)%XRRVEG
XIRRIG_FLUX=>DIAG_EVAP_ISBA_MODEL(KTO)%XIRRIG_FLUX
XGPP=>DIAG_EVAP_ISBA_MODEL(KTO)%XGPP
XRESP_AUTO=>DIAG_EVAP_ISBA_MODEL(KTO)%XRESP_AUTO
XRESP_ECO=>DIAG_EVAP_ISBA_MODEL(KTO)%XRESP_ECO
XDWG=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWG
XDWGI=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWGI
XDWR=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWR
XDSWE=>DIAG_EVAP_ISBA_MODEL(KTO)%XDSWE
XWATBUD=>DIAG_EVAP_ISBA_MODEL(KTO)%XWATBUD
XAVG_LEG=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEG
XAVG_LEGI=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEGI
XAVG_LEV=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEV
XAVG_LES=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LES
XAVG_LESL=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LESL
XAVG_LER=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LER
XAVG_LETR=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LETR
XAVG_EVAP=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_EVAP
XAVG_DRAIN=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DRAIN
XAVG_RUNOFF=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RUNOFF
XAVG_HORT=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_HORT
XAVG_DRIP=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DRIP
XAVG_MELT=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_MELT
XAVG_IFLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_IFLOOD
XAVG_PFLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_PFLOOD
XAVG_LE_FLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LE_FLOOD
XAVG_LEI_FLOOD=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEI_FLOOD
XAVG_RRVEG=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RRVEG
XAVG_IRRIG_FLUX=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_IRRIG_FLUX
XAVG_GPP=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_GPP
XAVG_RESP_AUTO=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RESP_AUTO
XAVG_RESP_ECO=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RESP_ECO
XRAINFALL=>DIAG_EVAP_ISBA_MODEL(KTO)%XRAINFALL
XSNOWFALL=>DIAG_EVAP_ISBA_MODEL(KTO)%XSNOWFALL
XAVG_DWG=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWG
XAVG_DWGI=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWGI
XAVG_DWR=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWR
XAVG_DSWE=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DSWE
XAVG_WATBUD=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_WATBUD
XRNC=>DIAG_EVAP_ISBA_MODEL(KTO)%XRNC
XHC=>DIAG_EVAP_ISBA_MODEL(KTO)%XHC
XLEC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEC
XLEIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEIC
XGFLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XGFLUXC
XLEGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEGC
XLEGIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEGIC
XLEVC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEVC
XLESC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLESC
XLESLC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLESLC
XLERC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLERC
XLETRC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLETRC
XEVAPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XEVAPC
XDRAINC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDRAINC
XRUNOFFC=>DIAG_EVAP_ISBA_MODEL(KTO)%XRUNOFFC
XHORTC=>DIAG_EVAP_ISBA_MODEL(KTO)%XHORTC
XDRIPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDRIPC
XMELTC=>DIAG_EVAP_ISBA_MODEL(KTO)%XMELTC
XIFLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XIFLOODC
XPFLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XPFLOODC
XLE_FLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLE_FLOODC
XLEI_FLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XLEI_FLOODC
XICEFLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XICEFLUXC
XRRVEGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XRRVEGC
XIRRIG_FLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XIRRIG_FLUXC
XGPPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XGPPC
XRESPC_AUTO=>DIAG_EVAP_ISBA_MODEL(KTO)%XRESPC_AUTO
XRESPC_ECO=>DIAG_EVAP_ISBA_MODEL(KTO)%XRESPC_ECO
XDWGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWGC
XDWGIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWGIC
XDWRC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDWRC
XDSWEC=>DIAG_EVAP_ISBA_MODEL(KTO)%XDSWEC
XWATBUDC=>DIAG_EVAP_ISBA_MODEL(KTO)%XWATBUDC
XAVG_RNC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RNC
XAVG_HC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_HC
XAVG_LEC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEC
XAVG_LEIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEIC
XAVG_GFLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_GFLUXC
XAVG_LEGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEGC
XAVG_LEGIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEGIC
XAVG_LEVC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEVC
XAVG_LESC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LESC
XAVG_LESLC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LESLC
XAVG_LERC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LERC
XAVG_LETRC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LETRC
XAVG_EVAPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_EVAPC
XAVG_DRAINC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DRAINC
XAVG_RUNOFFC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RUNOFFC
XAVG_HORTC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_HORTC
XAVG_DRIPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DRIPC
XAVG_MELTC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_MELTC
XAVG_IFLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_IFLOODC
XAVG_PFLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_PFLOODC
XAVG_LE_FLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LE_FLOODC
XAVG_LEI_FLOODC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_LEI_FLOODC
XAVG_ICEFLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_ICEFLUXC
XAVG_RRVEGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RRVEGC
XAVG_IRRIG_FLUXC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_IRRIG_FLUXC
XAVG_GPPC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_GPPC
XAVG_RESPC_AUTO=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RESPC_AUTO
XAVG_RESPC_ECO=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_RESPC_ECO
XRAINFALLC=>DIAG_EVAP_ISBA_MODEL(KTO)%XRAINFALLC
XSNOWFALLC=>DIAG_EVAP_ISBA_MODEL(KTO)%XSNOWFALLC
XAVG_DWGC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWGC
XAVG_DWGIC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWGIC
XAVG_DWRC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DWRC
XAVG_DSWEC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_DSWEC
XAVG_WATBUDC=>DIAG_EVAP_ISBA_MODEL(KTO)%XAVG_WATBUDC
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_EVAP_ISBA_GOTO_MODEL

SUBROUTINE DIAG_EVAP_ISBA_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_EVAP_ISBA_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEGI)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEV)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLES)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLESL)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLER)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLETR)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XEVAP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDRAIN)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRUNOFF)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XHORT)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRRVEG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XMELT)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XIFLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XPFLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLE_FLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEI_FLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDRIP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XIRRIG_FLUX)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XGPP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRESP_AUTO)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRESP_ECO)  
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWGI)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWR)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDSWE)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XWATBUD)  
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEGI)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEV)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LES)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LESL)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LER)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LETR)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_EVAP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DRAIN)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RUNOFF)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_HORT)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DRIP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_MELT)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_IFLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_PFLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LE_FLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEI_FLOOD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RRVEG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_IRRIG_FLUX)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_GPP)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RESP_AUTO)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RESP_ECO)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRAINFALL)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XSNOWFALL)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWG)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWGI)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWR)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DSWE)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_WATBUD)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRNC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XHC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XGFLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEGIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEVC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLESC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLESLC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLERC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLETRC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XEVAPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDRAINC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRUNOFFC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XHORTC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDRIPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XMELTC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XIFLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XPFLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLE_FLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XLEI_FLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XICEFLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRRVEGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XIRRIG_FLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XGPPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRESPC_AUTO)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRESPC_ECO) 
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWGIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDWRC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XDSWEC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XWATBUDC) 
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RNC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_HC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_GFLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEGIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEVC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LESC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LESLC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LERC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LETRC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_EVAPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DRAINC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RUNOFFC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_HORTC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DRIPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_MELTC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_IFLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_PFLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LE_FLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_LEI_FLOODC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_ICEFLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RRVEGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_IRRIG_FLUXC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_GPPC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RESPC_AUTO)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_RESPC_ECO)  
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XRAINFALLC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XSNOWFALLC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWGC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWGIC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DWRC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_DSWEC)
  NULLIFY(DIAG_EVAP_ISBA_MODEL(J)%XAVG_WATBUDC)  
ENDDO
DIAG_EVAP_ISBA_MODEL(:)%LSURF_EVAP_BUDGET=.FALSE.
DIAG_EVAP_ISBA_MODEL(:)%LSURF_BUDGETC=.FALSE.
DIAG_EVAP_ISBA_MODEL(:)%LRESET_BUDGETC=.FALSE.
DIAG_EVAP_ISBA_MODEL(:)%LWATER_BUDGET=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_EVAP_ISBA_ALLOC

SUBROUTINE DIAG_EVAP_ISBA_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_EVAP_ISBA_MODEL)) DEALLOCATE(DIAG_EVAP_ISBA_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_EVAP_ISBA_N:DIAG_EVAP_ISBA_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_EVAP_ISBA_DEALLO

END MODULE MODD_DIAG_EVAP_ISBA_n
