!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ############################
      MODULE MODD_DIAG_MISC_FLAKE_n
!     ############################
!
!!****  *MODD_DIAG_MISC_FLAKE - declaration of diagnostic variables for FLAKE scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/10/04
!
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_MISC_FLAKE_t
!------------------------------------------------------------------------------
!
  LOGICAL :: LWATER_PROFILE   ! flag for miscellaneous terms of FLake scheme
!
!* miscellaneous variables
!
  REAL, POINTER, DIMENSION(:) :: XZWAT_PROFILE ! depth of output levels (m) in namelist
  REAL, POINTER, DIMENSION(:)   :: XZW_PROFILE ! depth of output levels (m)
  REAL, POINTER, DIMENSION(:,:) :: XTW_PROFILE ! Water temperature in output levels (K)
!
!
!------------------------------------------------------------------------------
!

END TYPE DIAG_MISC_FLAKE_t

TYPE(DIAG_MISC_FLAKE_t), ALLOCATABLE, TARGET, SAVE :: DIAG_MISC_FLAKE_MODEL(:)

LOGICAL, POINTER :: LWATER_PROFILE=>NULL()
!$OMP THREADPRIVATE(LWATER_PROFILE)
REAL, POINTER, DIMENSION(:)   :: XZWAT_PROFILE=>NULL()
!$OMP THREADPRIVATE(XZWAT_PROFILE)
REAL, POINTER, DIMENSION(:)   :: XZW_PROFILE=>NULL()
!$OMP THREADPRIVATE(XZW_PROFILE)
REAL, POINTER, DIMENSION(:,:)   :: XTW_PROFILE=>NULL()
!$OMP THREADPRIVATE(XTW_PROFILE)

CONTAINS

SUBROUTINE DIAG_MISC_FLAKE_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_MISC_FLAKE_MODEL(KFROM)%XZWAT_PROFILE=>XZWAT_PROFILE
DIAG_MISC_FLAKE_MODEL(KFROM)%XZW_PROFILE=>XZW_PROFILE
DIAG_MISC_FLAKE_MODEL(KFROM)%XTW_PROFILE=>XTW_PROFILE
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_GOTO_MODEL',0,ZHOOK_HANDLE)
LWATER_PROFILE=>DIAG_MISC_FLAKE_MODEL(KTO)%LWATER_PROFILE
XZWAT_PROFILE=>DIAG_MISC_FLAKE_MODEL(KTO)%XZWAT_PROFILE
XZW_PROFILE=>DIAG_MISC_FLAKE_MODEL(KTO)%XZW_PROFILE
XTW_PROFILE=>DIAG_MISC_FLAKE_MODEL(KTO)%XTW_PROFILE
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_MISC_FLAKE_GOTO_MODEL

SUBROUTINE DIAG_MISC_FLAKE_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_MISC_FLAKE_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_MISC_FLAKE_MODEL(J)%XZWAT_PROFILE)
  NULLIFY(DIAG_MISC_FLAKE_MODEL(J)%XZW_PROFILE)
  NULLIFY(DIAG_MISC_FLAKE_MODEL(J)%XTW_PROFILE)
ENDDO
DIAG_MISC_FLAKE_MODEL(:)%LWATER_PROFILE=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_FLAKE_ALLOC

SUBROUTINE DIAG_MISC_FLAKE_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_MISC_FLAKE_MODEL)) DEALLOCATE(DIAG_MISC_FLAKE_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_FLAKE_N:DIAG_MISC_FLAKE_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_FLAKE_DEALLO

END MODULE MODD_DIAG_MISC_FLAKE_n
