!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_FLAKE_SBL_n
!     ################
!
!!****  *MODD_FLAKE_SBL_n - declaration of surface parameters for SBL
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/2006
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE FLAKE_SBL_t
!
  INTEGER                       :: NLVL ! number      of levels in SBL
  REAL, POINTER, DIMENSION(:,:) :: XZ   ! height of middle of each level grid(m)
  REAL, POINTER, DIMENSION(:,:) :: XU   ! wind        at each level in SBL   (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XT   ! temperature at each level in SBL   (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XQ   ! humidity    at each level in SBL   (kg/m3)
  REAL, POINTER, DIMENSION(:,:) :: XTKE ! Tke         at each level in SBL   (m2/s2)
  REAL, POINTER, DIMENSION(:)   :: XLMO ! Monin-Obhukov length               (m)
  REAL, POINTER, DIMENSION(:,:) :: XP   ! pressure    at each level in SBL   (kg/m3)
!
  REAL, POINTER, DIMENSION(:,:) :: XDZ  ! depth       of each level in SBL   (m)
  REAL, POINTER, DIMENSION(:,:) :: XZF  ! height of bottom of each level grid(m)
  REAL, POINTER, DIMENSION(:,:) :: XDZF ! depth between  each level in SBL   (m)
!
END TYPE FLAKE_SBL_t

TYPE(FLAKE_SBL_t), ALLOCATABLE, TARGET, SAVE :: FLAKE_SBL_MODEL(:)

INTEGER, POINTER :: NLVL=>NULL()
!$OMP THREADPRIVATE(NLVL)
REAL, POINTER, DIMENSION(:,:) :: XZ=>NULL()
!$OMP THREADPRIVATE(XZ)
REAL, POINTER, DIMENSION(:,:) :: XU=>NULL()
!$OMP THREADPRIVATE(XU)
REAL, POINTER, DIMENSION(:,:) :: XT=>NULL()
!$OMP THREADPRIVATE(XT)
REAL, POINTER, DIMENSION(:,:) :: XQ=>NULL()
!$OMP THREADPRIVATE(XQ)
REAL, POINTER, DIMENSION(:,:) :: XTKE=>NULL()
!$OMP THREADPRIVATE(XTKE)
REAL, POINTER, DIMENSION(:)   :: XLMO=>NULL()
!$OMP THREADPRIVATE(XLMO)
REAL, POINTER, DIMENSION(:,:) :: XP=>NULL()
!$OMP THREADPRIVATE(XP)
REAL, POINTER, DIMENSION(:,:) :: XDZ=>NULL()
!$OMP THREADPRIVATE(XDZ)
REAL, POINTER, DIMENSION(:,:) :: XZF=>NULL()
!$OMP THREADPRIVATE(XZF)
REAL, POINTER, DIMENSION(:,:) :: XDZF=>NULL()
!$OMP THREADPRIVATE(XDZF)

CONTAINS

SUBROUTINE FLAKE_SBL_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
FLAKE_SBL_MODEL(KFROM)%XZ=>XZ
FLAKE_SBL_MODEL(KFROM)%XU=>XU
FLAKE_SBL_MODEL(KFROM)%XT=>XT
FLAKE_SBL_MODEL(KFROM)%XQ=>XQ
FLAKE_SBL_MODEL(KFROM)%XTKE=>XTKE
FLAKE_SBL_MODEL(KFROM)%XLMO=>XLMO
FLAKE_SBL_MODEL(KFROM)%XP=>XP
FLAKE_SBL_MODEL(KFROM)%XDZ=>XDZ
FLAKE_SBL_MODEL(KFROM)%XZF=>XZF
FLAKE_SBL_MODEL(KFROM)%XDZF=>XDZF
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_FLAKE_SBL_N:FLAKE_SBL_GOTO_MODEL',0,ZHOOK_HANDLE)
NLVL=>FLAKE_SBL_MODEL(KTO)%NLVL
XZ=>FLAKE_SBL_MODEL(KTO)%XZ
XU=>FLAKE_SBL_MODEL(KTO)%XU
XT=>FLAKE_SBL_MODEL(KTO)%XT
XQ=>FLAKE_SBL_MODEL(KTO)%XQ
XTKE=>FLAKE_SBL_MODEL(KTO)%XTKE
XLMO=>FLAKE_SBL_MODEL(KTO)%XLMO
XP=>FLAKE_SBL_MODEL(KTO)%XP
XDZ=>FLAKE_SBL_MODEL(KTO)%XDZ
XZF=>FLAKE_SBL_MODEL(KTO)%XZF
XDZF=>FLAKE_SBL_MODEL(KTO)%XDZF
IF (LHOOK) CALL DR_HOOK('MODD_FLAKE_SBL_N:FLAKE_SBL_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE FLAKE_SBL_GOTO_MODEL

SUBROUTINE FLAKE_SBL_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_SBL_N:FLAKE_SBL_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(FLAKE_SBL_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(FLAKE_SBL_MODEL(J)%XZ)
  NULLIFY(FLAKE_SBL_MODEL(J)%XU)
  NULLIFY(FLAKE_SBL_MODEL(J)%XT)
  NULLIFY(FLAKE_SBL_MODEL(J)%XQ)
  NULLIFY(FLAKE_SBL_MODEL(J)%XTKE)
  NULLIFY(FLAKE_SBL_MODEL(J)%XLMO)
  NULLIFY(FLAKE_SBL_MODEL(J)%XP)
  NULLIFY(FLAKE_SBL_MODEL(J)%XDZ)
  NULLIFY(FLAKE_SBL_MODEL(J)%XZF)
  NULLIFY(FLAKE_SBL_MODEL(J)%XDZF)
ENDDO
FLAKE_SBL_MODEL(:)%NLVL=0
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_SBL_N:FLAKE_SBL_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE FLAKE_SBL_ALLOC

SUBROUTINE FLAKE_SBL_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_SBL_N:FLAKE_SBL_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(FLAKE_SBL_MODEL)) DEALLOCATE(FLAKE_SBL_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_FLAKE_SBL_N:FLAKE_SBL_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE FLAKE_SBL_DEALLO

END MODULE MODD_FLAKE_SBL_n
